@@ -1,22 +1,24 @@
-use clap::ValueEnum;
-use owo_colors::OwoColorize;
-use serde::Serialize;
-use std::collections::BTreeSet;
 use std::fmt::Write;
 use std::path::PathBuf;
 
+use clap::ValueEnum;
+use owo_colors::OwoColorize;
+use serde::Serialize;
+
 use crate::cli::reporter::HookInitReporter;
+use crate::cli::run::Selectors;
 use crate::cli::{ExitStatus, ListOutputFormat};
 use crate::config::{Language, Stage};
 use crate::fs::CWD;
 use crate::hook;
 use crate::printer::Printer;
 use crate::store::STORE;
-use crate::workspace::Project;
+use crate::workspace::Workspace;
 
 #[derive(Serialize)]
 struct SerializableHook {
     id: String,
+    full_id: String,
     name: String,
     alias: String,
     language: Language,
@@ -26,37 +28,43 @@ struct SerializableHook {
 
 pub(crate) async fn list(
     config: Option<PathBuf>,
-    verbose: bool,
-    hook_ids: Vec<String>,
+    includes: Vec<String>,
+    skips: Vec<String>,
     hook_stage: Option<Stage>,
     language: Option<Language>,
     output_format: ListOutputFormat,
+    refresh: bool,
+    verbose: bool,
     printer: Printer,
 ) -> anyhow::Result<ExitStatus> {
-    let mut project = Project::from_config_file_or_directory(config, &CWD)?;
+    let workspace_root = Workspace::find_root(config.as_deref(), &CWD)?;
+    let selectors = Selectors::load(&includes, &skips, &workspace_root)?;
+    let mut workspace = Workspace::discover(workspace_root, config, Some(&selectors), refresh)?;
+
     let store = STORE.as_ref()?;
-
     let reporter = HookInitReporter::from(printer);
-
     let lock = store.lock_async().await?;
-    let hooks = project.init_hooks(store, Some(&reporter)).await?;
+    let hooks = workspace.init_hooks(store, Some(&reporter)).await?;
+
     drop(lock);
 
-    let hook_ids = hook_ids.into_iter().collect::<BTreeSet<_>>();
-    let hooks: Vec<_> = hooks
+    let filtered_hooks: Vec<_> = hooks
         .into_iter()
-        .filter(|h| hook_ids.is_empty() || hook_ids.contains(&h.id) || hook_ids.contains(&h.alias))
+        .filter(|h| selectors.matches_hook(h))
         .filter(|h| hook_stage.is_none_or(|hook_stage| h.stages.contains(hook_stage)))
         .filter(|h| language.is_none_or(|lang| h.language == lang))
         .collect();
 
+    selectors.report_unused();
+
     match output_format {
         ListOutputFormat::Text => {
             if verbose {
                 // TODO: show repo path and environment path (if installed)
-                for hook in &hooks {
-                    writeln!(printer.stdout(), "{}", hook.id.bold())?;
+                for hook in &filtered_hooks {
+                    writeln!(printer.stdout(), "{}", hook.full_id().bold())?;
 
+                    writeln!(printer.stdout(), "  {} {}", "ID:".bold().cyan(), hook.id)?;
                     if !hook.alias.is_empty() && hook.alias != hook.id {
                         writeln!(
                             printer.stdout(),
@@ -94,21 +102,25 @@ pub(crate) async fn list(
                     writeln!(printer.stdout())?;
                 }
             } else {
-                for hook in &hooks {
-                    writeln!(printer.stdout(), "{}", hook.id)?;
+                // TODO: add project prefix to hook id
+                for hook in &filtered_hooks {
+                    writeln!(printer.stdout(), "{}", hook.full_id())?;
                 }
             }
         }
         ListOutputFormat::Json => {
-            let serializable_hooks: Vec<_> = hooks
+            let serializable_hooks: Vec<_> = filtered_hooks
                 .into_iter()
                 .map(|h| {
+                    let id = h.id.to_string();
+                    let full_id = h.full_id();
                     let stages = match h.stages {
                         hook::Stages::All => Stage::value_variants().to_vec(),
                         hook::Stages::Some(s) => s.into_iter().collect(),
                     };
                     SerializableHook {
-                        id: h.id,
+                        id,
+                        full_id,
                         name: h.name,
                         alias: h.alias,
                         language: h.language,
