@@ -1,4 +1,3 @@
-use std::collections::HashMap;
 use std::fmt::Display;
 use std::ops::{Deref, RangeInclusive};
 use std::path::Path;
@@ -6,16 +5,16 @@ use std::str::FromStr;
 use std::sync::OnceLock;
 
 use anyhow::Result;
+use constants::{ALT_CONFIG_FILE, CONFIG_FILE};
 use fancy_regex::{self as regex, Regex};
+use rustc_hash::FxHashMap;
 use serde::{Deserialize, Deserializer, Serialize, Serializer};
 
 use crate::fs::Simplified;
+use crate::identify;
 use crate::version;
 use crate::warn_user;
 
-pub const CONFIG_FILE: &str = ".pre-commit-config.yaml";
-pub const MANIFEST_FILE: &str = ".pre-commit-hooks.yaml";
-
 #[derive(Clone)]
 pub struct SerdeRegex(Regex);
 
@@ -278,7 +277,7 @@ pub struct Config {
     /// Default is `[pre-commit]`.
     pub default_install_hook_types: Option<Vec<HookType>>,
     /// A mapping from language to the default `language_version`.
-    pub default_language_version: Option<HashMap<Language, String>>,
+    pub default_language_version: Option<FxHashMap<Language, String>>,
     /// A configuration-wide default for the stages property of hooks.
     /// Default to all stages.
     pub default_stages: Option<Vec<Stage>>,
@@ -293,7 +292,7 @@ pub struct Config {
     #[serde(deserialize_with = "deserialize_minimum_version", default)]
     pub minimum_prek_version: Option<String>,
     /// Configuration for pre-commit.ci service.
-    pub ci: Option<HashMap<String, serde_yaml::Value>>,
+    pub ci: Option<FxHashMap<String, serde_yaml::Value>>,
 }
 
 #[derive(Debug, Clone, Serialize)]
@@ -354,12 +353,15 @@ pub struct HookOptions {
     pub exclude: Option<SerdeRegex>,
     /// List of file types to run on (AND).
     /// Default is `[file]`, which matches all files.
+    #[serde(deserialize_with = "deserialize_and_validate_tags", default)]
     pub types: Option<Vec<String>>,
     /// List of file types to run on (OR).
     /// Default is `[]`.
+    #[serde(deserialize_with = "deserialize_and_validate_tags", default)]
     pub types_or: Option<Vec<String>>,
     /// List of file types to exclude.
     /// Default is `[]`.
+    #[serde(deserialize_with = "deserialize_and_validate_tags", default)]
     pub exclude_types: Option<Vec<String>>,
     /// Not documented in the official docs.
     pub additional_dependencies: Option<Vec<String>>,
@@ -521,9 +523,13 @@ impl<'de> Deserialize<'de> for MetaHook {
                 entry: String::new(),
                 options: HookOptions {
                     files: Some(
-                        Regex::new(&format!("^{}$", regex::escape(CONFIG_FILE),))
-                            .map(SerdeRegex)
-                            .unwrap(),
+                        Regex::new(&format!(
+                            "^{}|{}$",
+                            regex::escape(CONFIG_FILE),
+                            regex::escape(ALT_CONFIG_FILE)
+                        ))
+                        .map(SerdeRegex)
+                        .unwrap(),
                     ),
                     ..Default::default()
                 },
@@ -535,9 +541,13 @@ impl<'de> Deserialize<'de> for MetaHook {
                 entry: String::new(),
                 options: HookOptions {
                     files: Some(
-                        Regex::new(&format!("^{}$", regex::escape(CONFIG_FILE),))
-                            .map(SerdeRegex)
-                            .unwrap(),
+                        Regex::new(&format!(
+                            "^{}|{}$",
+                            regex::escape(CONFIG_FILE),
+                            regex::escape(ALT_CONFIG_FILE)
+                        ))
+                        .map(SerdeRegex)
+                        .unwrap(),
                     ),
                     ..Default::default()
                 },
@@ -802,6 +812,24 @@ fn looks_like_sha(s: &str) -> bool {
     re.is_match(s).unwrap_or(false)
 }
 
+/// Deserializes a vector of strings and validates that each is a known file type tag.
+fn deserialize_and_validate_tags<'de, D>(deserializer: D) -> Result<Option<Vec<String>>, D::Error>
+where
+    D: Deserializer<'de>,
+{
+    let tags_opt: Option<Vec<String>> = Option::deserialize(deserializer)?;
+    if let Some(tags) = &tags_opt {
+        let all_tags = identify::all_tags();
+        for tag in tags {
+            if !all_tags.contains(tag.as_str()) {
+                let msg = format!("Type tag \"{tag}\" is not recognized. Try upgrading prek");
+                return Err(serde::de::Error::custom(msg));
+            }
+        }
+    }
+    Ok(tags_opt)
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -1144,7 +1172,7 @@ mod tests {
                                             alias: None,
                                             files: Some(
                                                 SerdeRegex(
-                                                    "^\\.pre-commit-config\\.yaml$",
+                                                    "^\\.pre-commit-config\\.yaml|\\.pre-commit-config\\.yml$",
                                                 ),
                                             ),
                                             exclude: None,
@@ -1176,7 +1204,7 @@ mod tests {
                                             alias: None,
                                             files: Some(
                                                 SerdeRegex(
-                                                    "^\\.pre-commit-config\\.yaml$",
+                                                    "^\\.pre-commit-config\\.yaml|\\.pre-commit-config\\.yml$",
                                                 ),
                                             ),
                                             exclude: None,
@@ -1451,4 +1479,104 @@ mod tests {
         let result = serde_yaml::from_str::<Config>(yaml);
         assert!(result.is_err());
     }
+
+    #[test]
+    fn test_validate_type_tags() {
+        // Valid tags should parse successfully
+        let yaml_valid = r"
+            repos:
+              - repo: local
+                hooks:
+                  - id: my-hook
+                    name: My Hook
+                    entry: echo
+                    language: system
+                    types: [python, file]
+                    types_or: [text, binary]
+                    exclude_types: [symlink]
+        ";
+        let result = serde_yaml::from_str::<Config>(yaml_valid);
+        assert!(result.is_ok(), "Should parse valid tags successfully");
+
+        // Empty lists and missing keys should also be fine
+        let yaml_empty = r"
+            repos:
+              - repo: local
+                hooks:
+                  - id: my-hook
+                    name: My Hook
+                    entry: echo
+                    language: system
+                    types: []
+                    exclude_types: []
+                    # types_or is missing, which is also valid
+        ";
+        let result_empty = serde_yaml::from_str::<Config>(yaml_empty);
+        assert!(
+            result_empty.is_ok(),
+            "Should parse empty/missing tags successfully"
+        );
+
+        // Invalid tag in 'types' should fail
+        let yaml_invalid_types = r"
+            repos:
+              - repo: local
+                hooks:
+                  - id: my-hook
+                    name: My Hook
+                    entry: echo
+                    language: system
+                    types: [pythoon] # Deliberate typo
+        ";
+        let result_invalid_types = serde_yaml::from_str::<Config>(yaml_invalid_types);
+        assert!(result_invalid_types.is_err());
+
+        assert!(
+            result_invalid_types
+                .unwrap_err()
+                .to_string()
+                .contains("Type tag \"pythoon\" is not recognized")
+        );
+
+        // Invalid tag in 'types_or' should fail
+        let yaml_invalid_types_or = r"
+            repos:
+              - repo: local
+                hooks:
+                  - id: my-hook
+                    name: My Hook
+                    entry: echo
+                    language: system
+                    types_or: [invalidtag]
+        ";
+        let result_invalid_types_or = serde_yaml::from_str::<Config>(yaml_invalid_types_or);
+        assert!(result_invalid_types_or.is_err());
+        assert!(
+            result_invalid_types_or
+                .unwrap_err()
+                .to_string()
+                .contains("Type tag \"invalidtag\" is not recognized")
+        );
+
+        // Invalid tag in 'exclude_types' should fail
+        let yaml_invalid_exclude_types = r"
+            repos:
+              - repo: local
+                hooks:
+                  - id: my-hook
+                    name: My Hook
+                    entry: echo
+                    language: system
+                    exclude_types: [not-a-real-tag]
+        ";
+        let result_invalid_exclude_types =
+            serde_yaml::from_str::<Config>(yaml_invalid_exclude_types);
+        assert!(result_invalid_exclude_types.is_err());
+        assert!(
+            result_invalid_exclude_types
+                .unwrap_err()
+                .to_string()
+                .contains("Type tag \"not-a-real-tag\" is not recognized")
+        );
+    }
 }
