@@ -4,7 +4,6 @@ use std::sync::Arc;
 
 use anyhow::{Context, Result};
 use constants::env_vars::EnvVars;
-use rustc_hash::FxHashSet;
 use tokio::io::AsyncWriteExt;
 use tracing::debug;
 
@@ -102,7 +101,7 @@ impl LanguageImpl for Pygrep {
             python = Some(installed);
         } else {
             // 2) If not found, try to find a system installed Python (system or system uv managed).
-            debug!("No Python interpreter found, trying to find a system installed one");
+            debug!("No managed Python interpreter found, trying to find a system installed one");
             let mut output = uv
                 .cmd("uv python find", store)
                 .arg("python")
@@ -145,7 +144,11 @@ impl LanguageImpl for Pygrep {
             anyhow::bail!("Failed to find or install a Python interpreter for `pygrep`.");
         };
 
-        let mut info = InstallInfo::new(hook.language, FxHashSet::default(), &store.hooks_dir());
+        let mut info = InstallInfo::new(
+            hook.language,
+            hook.dependencies().clone(),
+            &store.hooks_dir(),
+        );
         info.with_toolchain(python);
 
         fs_err::tokio::create_dir_all(&info.env_path).await?;
@@ -165,7 +168,7 @@ impl LanguageImpl for Pygrep {
     async fn run(
         &self,
         hook: &InstalledHook,
-        filenames: &[&String],
+        filenames: &[&Path],
         store: &Store,
     ) -> Result<(i32, Vec<u8>)> {
         let InstalledHook::Installed { info, .. } = hook else {
@@ -182,6 +185,7 @@ impl LanguageImpl for Pygrep {
 
         let args = Args::parse(&hook.args).context("Failed to parse `args`")?;
         let mut cmd = Cmd::new(&info.toolchain, "python script")
+            .current_dir(hook.work_dir())
             .arg("-I") // Isolate mode.
             .arg("-B") // Don't write bytecode.
             .arg(py_script.path())
@@ -196,11 +200,13 @@ impl LanguageImpl for Pygrep {
 
         let mut stdin = cmd.stdin.take().context("Failed to take stdin")?;
         // TODO: avoid this clone if possible.
-        let filenames: Vec<_> = filenames.iter().map(ToString::to_string).collect();
+        let filenames: Vec<_> = filenames.iter().map(PathBuf::from).collect();
 
         let write_task = tokio::spawn(async move {
             for filename in filenames {
-                stdin.write_all(format!("{filename}\n").as_bytes()).await?;
+                stdin
+                    .write_all(format!("{}\n", filename.display()).as_bytes())
+                    .await?;
             }
             let _ = stdin.shutdown().await;
             anyhow::Ok(())
