@@ -1,8 +1,10 @@
+mod pep723;
 #[allow(clippy::module_inception)]
 mod python;
 mod uv;
 mod version;
 
+pub(crate) use pep723::extract_pep723_metadata;
 pub(crate) use python::Python;
 pub(crate) use python::python_exec;
 pub(crate) use uv::Uv;
