@@ -1,9 +1,9 @@
+use std::path::Path;
 use std::sync::Arc;
 
 use anyhow::Result;
 
 use crate::cli::reporter::HookInstallReporter;
-use crate::fs::CWD;
 use crate::hook::Hook;
 use crate::hook::InstalledHook;
 use crate::languages::{LanguageImpl, resolve_command};
@@ -31,22 +31,23 @@ impl LanguageImpl for Script {
     async fn run(
         &self,
         hook: &InstalledHook,
-        filenames: &[&String],
+        filenames: &[&Path],
         _store: &Store,
     ) -> Result<(i32, Vec<u8>)> {
         // For `language: script`, the `entry[0]` is a script path.
         // For remote hooks, the path is relative to the repo root.
         // For local hooks, the path is relative to the current working directory.
-        // TODO: In workspace mode, local hooks should be relative to the project root.
-        let repo_path = hook.repo_path().unwrap_or_else(|| CWD.as_path());
+
+        let repo_path = hook.repo_path().unwrap_or(hook.work_dir());
         let mut split = hook.entry.split()?;
 
         let cmd = repo_path.join(&split[0]);
         split[0] = cmd.to_string_lossy().to_string();
         let entry = resolve_command(split, None);
 
-        let run = async move |batch: &[&String]| {
+        let run = async move |batch: &[&Path]| {
             let mut output = Cmd::new(&entry[0], "run script command")
+                .current_dir(hook.work_dir())
                 .args(&entry[1..])
                 .args(&hook.args)
                 .args(batch)
